#!/usr/bin/perl -w

use Test::More 'no_plan';

package Catch;

sub TIEHANDLE {
    my($class, $var) = @_;
    return bless { var => $var }, $class;
}

sub PRINT  {
    my($self) = shift;
    ${'main::'.$self->{var}} .= join '', @_;
}

sub OPEN  {}    # XXX Hackery in case the user redirects
sub CLOSE {}    # XXX STDERR/STDOUT.  This is not the behavior we want.

sub READ {}
sub READLINE {}
sub GETC {}
sub BINMODE {}

my $Original_File = 'lib/Inca/ReporterManager.pm';

package main;

# pre-5.8.0's warns aren't caught by a tied STDERR.
$SIG{__WARN__} = sub { $main::_STDERR_ .= join '', @_; };
tie *STDOUT, 'Catch', '_STDOUT_' or die $!;
tie *STDERR, 'Catch', '_STDERR_' or die $!;

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 145 lib/Inca/ReporterManager.pm

  use Test::Exception;
  use Inca::ReporterManager;
  lives_ok { new Inca::ReporterManager() } 'object created';


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 211 lib/Inca/ReporterManager.pm

  untie *STDOUT;
  untie *STDERR;

  use strict;
  use warnings;
  use Inca::Logger;
  use Inca::Net::MockDepot;
  use Inca::ReporterManager;
  use Inca::ReporterManager::ReporterCache;
  use Inca::Suite;

  Inca::Logger->screen_init( "ERROR" );

  my $PORT = 7834;
  my $CREDS = { cert => "t/certs/client_ca1cert.pem",
                key => "t/certs/client_ca1keynoenc.pem",
                passphrase => undef,
                trusted => "t/certs/trusted" };

  sub testRM {
    my $numReportsExpected = shift;
    my @suite_files = @_;

    my $rc = new Inca::ReporterManager::ReporterCache( "t",
      errorReporterPath => "bin/inca-null-reporter"
    );
  
    my $rm = new Inca::ReporterManager();
    $rm->setId( "resourceA" );
    $rm->setCheckPeriod( 1 );
    $rm->setReporterCache( $rc );
    $rm->setDepots( "incas://localhost:$PORT" );
    $rm->setTmpDirectory( "/tmp" );
    $rm->setCredentials( $CREDS );
    $rm->setLogger( "/tmp/rm.log" );
    $rm->setRimPath( "sbin/reporter-instance-manager" );
    pipe( READ, WRITE );
    if ( ! fork() ) {
      for my $suite_file ( @suite_files ) {
        my $suite = new Inca::Suite();
        $suite->read( $suite_file );
        $suite->resolveReporters( $rc, "incas://localhost:8787" );
        $rm->dispatch( $suite );
        $rm->start();
      }
      <READ>;
      $rm->stop();
      exit;
    }
    my $depot = new Inca::Net::MockDepot;
    my $numReports = $depot->readReportsFromSSL(
      $numReportsExpected, $PORT, "ca1", "t/certs/trusted"
    );
    print WRITE "done\n";
    my @reports = $depot->getReports();
    return ( $numReports, @reports );
  }

  my ( $numReports, @reports ) = testRM( 2, "t/suite.xml" );
  is( $numReports, 2, "2 reports from suite file" );

  # check var_suite
  ( $numReports, @reports ) = testRM( 1, "t/var_suite.xml" );
  is( $numReports, 1, "1 report from var_suite file" );
  like( $reports[0], qr/-verbose=0 -help=no -var1=$ENV{HOME} -var2=/m, 
        "looks like context passed as arg");
 
  # test file with cron
  ( $numReports, @reports ) = testRM( 4, "t/cron_suite.xml" );
  my $num_gcc_reports = grep( />gcc</m, @reports );
  is( $num_gcc_reports, 2, "2 gcc reports found" );
  my $num_ssl_reports = grep( />openssl</m, @reports );
  is( $num_ssl_reports, 2, "2 ssl reports found" );

  # test add/delete capability
  ( $numReports, @reports ) = testRM( 2, "t/cron_suite.xml", "t/delete_cron_suite.xml"  );
  $num_gcc_reports = grep( />gcc</m, @reports );
  $num_ssl_reports = grep( />openssl</m, @reports );
  is( $num_gcc_reports, 0, "0 gcc reports found" );
  is( $num_ssl_reports, 2, "2 ssl reports found" );

  # test cron with 2 groups
  ( $numReports, @reports ) = testRM( 2, "t/cron_suite2.xml" );
  $num_gcc_reports = grep( />gcc</m, @reports );
  is( $num_gcc_reports, 1, "one gcc report found" );
  $num_ssl_reports = grep( />openssl</m, @reports );
  is( $num_ssl_reports, 1, "one ssl report found" );
  my @gmt = grep( /gmt/, @reports );
  my ($minute1) = $gmt[0] =~ /(\d\d):\d\dZ/g;
  my ($minute2) = $gmt[1] =~ /(\d\d):\d\dZ/g;
  is( $minute2 - $minute1, 1, "time difference is one minute" );

  # test mixed
  ( $numReports, @reports ) = testRM( 2, "t/mixed_suite.xml" );
  $num_gcc_reports = grep( />gcc</m, @reports );
  is( $num_gcc_reports, 1, "one gcc report found for mixed suite" );
  $num_ssl_reports = grep( />openssl</m, @reports );
  is( $num_ssl_reports, 1, "one ssl report found for mixed suite" );


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 541 lib/Inca/ReporterManager.pm

  use Inca::ReporterManager();
  my $sched = new Inca::ReporterManager();
  ok( ! $sched->hasTmpDirectory(), 'hasTmpDirectory for false' );
  $sched->setTmpDirectory( "/tmp" );
  ok( $sched->hasTmpDirectory(), 'hasTmpDirectory for true' );


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 580 lib/Inca/ReporterManager.pm


  use Inca::ReporterManager;

  my $scheduler = new Inca::ReporterManager( checkPeriod => 3 );
  is( $scheduler->getCheckPeriod(), 3, 'set check period from constructor' );
  $scheduler->setCheckPeriod( 10 );
  is( $scheduler->getCheckPeriod(), 10, 'set check period' );
  $scheduler = new Inca::ReporterManager();
  is( $scheduler->getCheckPeriod(), 2, 'default check period is 2' );


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 623 lib/Inca/ReporterManager.pm

  use Inca::ReporterManager;
  use Inca::ReporterManager::ReporterCache;
  use Test::Exception;

  my $credentials = { cert => 't/cert.pem', key => 't/key.pem',
                      passphrase => 'secret', trusted => 't/trusted' };
  my $credentials_new = { cert => 't/new/cert.pem', key => 't/key.pem',
                      passphrase => 'supersecret', trusted => 't/trusted' };
  my $scheduler = new Inca::ReporterManager( 
                  credentials => $credentials );
  ok( eq_hash($scheduler->getCredentials(), $credentials), 
          'set credentials worked from constructor' );
  $scheduler->setCredentials( $credentials_new );
  ok( eq_hash($scheduler->getCredentials(), $credentials_new), 
          'set/get credentials worked' );


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 674 lib/Inca/ReporterManager.pm

  use Inca::ReporterManager;
  use Test::Exception;

  my $scheduler = new Inca::ReporterManager(depotURIs=>["cuzco.sdsc.edu:8234"]);
  my @depots = $scheduler->getDepots();
  is( $depots[0], "cuzco.sdsc.edu:8234", 'Set depots works from constructor' );
  $scheduler->setDepots( qw(inca.sdsc.edu:8235 inca.sdsc.edu:8236) );
  @depots = $scheduler->getDepots();
  is( $depots[0], "inca.sdsc.edu:8235", 'Set depots works from function (1)' );
  is( $depots[1], "inca.sdsc.edu:8236", 'Set depots works from function (2)' );


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 720 lib/Inca/ReporterManager.pm

  use Inca::ReporterManager;

  my $rm = new Inca::ReporterManager( id => "resourceA" );
  is( $rm->getId(), "resourceA", 'set id from constructor' );
  $rm->setId( "resourceB" );
  is( $rm->getId(), "resourceB", 'set id' );
  $rm = new Inca::ReporterManager( );
  is( $rm->getId(), undef, 'default id is undef' );


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 777 lib/Inca/ReporterManager.pm

  use Inca::ReporterManager;
  use Inca::ReporterManager::ReporterCache;
  use Test::Exception;
  use Cwd;

  my $rc = new Inca::ReporterManager::ReporterCache( "t" );
  my $rc_new = new Inca::ReporterManager::ReporterCache( "/tmp" );
  my $scheduler = new Inca::ReporterManager( reporterCache => $rc );
  is( $scheduler->getReporterCache->getLocation(), getcwd() . "/t",
          'set reporter admin worked from constructor' );
  $scheduler->setReporterCache( $rc_new );
  is( $scheduler->getReporterCache->getLocation(), "/tmp",
          'set reporter admin worked from set/get functions' );


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 825 lib/Inca/ReporterManager.pm

  use Inca::ReporterManager;
  use Test::Exception;

  my $sched = new Inca::ReporterManager(); 
  is( $sched->getRimPath(), undef, 'default rim_path set' );
  dies_ok { $sched->setRimPath( "bin/reporter-instance-manager" ) }
          'setRimPath fails when not executable';
  $sched->setRimPath( "sbin/reporter-instance-manager" );
  is( $sched->getRimPath(), "sbin/reporter-instance-manager", 
      'set/getRimPath work' );


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 873 lib/Inca/ReporterManager.pm

  use Inca::ReporterManager;
  use Test::Exception;

  my $sched = new Inca::ReporterManager(); 
  is( $sched->getTmpDirectory(), undef, 'default tmp set' );
  $sched->setTmpDirectory( "/scratch" );
  is( $sched->getTmpDirectory(), "/scratch", 'set/getTmpDirectory work' );


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 899 lib/Inca/ReporterManager.pm

  untie *STDOUT;
  untie *STDERR;

  use Inca::ReporterManager;
  use Inca::ReporterManager::ReporterCache;
  use Inca::Suite;

  my $rc = new Inca::ReporterManager::ReporterCache( "t",
    errorReporterPath => "bin/inca-null-reporter"
  );

  my $cron_suite = new Inca::Suite();
  $cron_suite->read( "t/cron_suite.xml" );
  $cron_suite->resolveReporters( $rc, "incas://localhost:8787" );
  $rm = new Inca::ReporterManager();
  $rm->setId( "resourceA" );
  $rm->setCheckPeriod( 1 );
  $rm->setReporterCache( $rc );
  $rm->setDepots( "file:./depot.tmp.$$" );
  $rm->setTmpDirectory( "/tmp" );
  $rm->dispatch( $cron_suite );
  ok( $rm->start(), "rm started" );
  ok( $rm->stop(), "rm stopped" );


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 1034 lib/Inca/ReporterManager.pm

  use Cwd;
  use Inca::ReporterManager;
  use Inca::ReporterManager::ReporterCache;

  my $rm = new Inca::ReporterManager();
  my $rc = new Inca::ReporterManager::ReporterCache( "var/cache" );
  $rm->setReporterCache( $rc );
  my $file = "/tmp/incatest.$$";
  `touch $file`;
  my $result = $rm->storePackage( 
    "Inca::Reporter",
    "Reporter.pm",
    "1.5",
    "lib/perl",
    undef,
    undef,
    $file 
  );
  is( $rm->getReporterCache()->getPath( 
        "Inca::Reporter", "1.5"
      ),
      getcwd() . "/var/cache/lib/perl/Reporter.pm",
      "getPath returned okay for Reporter.pm" );
  `rm -fr var/cache`;


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 1096 lib/Inca/ReporterManager.pm

  use Inca::ReporterManager;
  use Inca::ReporterManager::ReporterCache;

  # create scheduler object
  my $scheduler = new Inca::ReporterManager();
  my $rc = new Inca::ReporterManager::ReporterCache( "t" );
  $scheduler->setReporterCache( $rc );

  # test for sequential
  my $scheduler_object = $scheduler->_createScheduler(
                           "Inca::ReporterManager::Scheduler::Sequential" );
  isa_ok( $scheduler_object, "Inca::ReporterManager::Scheduler::Sequential",
          '_createScheduler returns sequential object' );

  # test for cron
  $scheduler_object = $scheduler->_createScheduler(
                           "Inca::ReporterManager::Scheduler::Cron" );
  isa_ok( $scheduler_object, "Inca::ReporterManager::Scheduler::Cron",
          '_createScheduler returns cron object' );


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 1164 lib/Inca/ReporterManager.pm

  tie *STDOUT, 'Catch', '_STDOUT_' or die $!;
  tie *STDERR, 'Catch', '_STDERR_' or die $!;

  use Inca::Logger;
  use Inca::ReporterManager;
  use Inca::ReporterManager::ReporterCache;

  Inca::Logger->screen_init( "ERROR" );

  # test for sequential
  my $rc = new Inca::ReporterManager::ReporterCache( "t" );
  my $scheduler = new Inca::ReporterManager();
  $scheduler->setReporterCache( $rc );
  my $scheduler_object = $scheduler->_getScheduler( "sequential" );
  isa_ok( $scheduler_object, "Inca::ReporterManager::Scheduler::Sequential" );

  # test for non-existent scheduler
  $scheduler_object = $scheduler->_getScheduler( "notsched" );
  is( $scheduler_object, undef, 'fails when not a scheduler' );
  like( $_STDERR_, qr/Unknown scheduler type/, "warns correctly when not a scheduler" );


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

{
    undef $main::_STDOUT_;
    undef $main::_STDERR_;
#line 1228 lib/Inca/ReporterManager.pm

  use Inca::ReporterManager;

  is( Inca::ReporterManager->_getSchedulerClassName('sequential'),
      "Inca::ReporterManager::Scheduler::Sequential",
      '_getSchedulerClassName returns correct class for sequential' );
  is( Inca::ReporterManager->_getSchedulerClassName('cron'),
      "Inca::ReporterManager::Scheduler::Cron",
      '_getSchedulerClassName returns correct class for cron' );


    undef $main::_STDOUT_;
    undef $main::_STDERR_;
}

    undef $main::_STDOUT_;
    undef $main::_STDERR_;
eval q{
  my $example = sub {
    local $^W = 0;

#line 12 lib/Inca/ReporterManager.pm

  use Inca::ReporterManager;
  use Inca::Suite;
  use Inca::ReporterManager::ReporterCache;
  my $suite = new Inca::Suite();
  $suite->read( "suite.xml" );
  my $scheduler = new Inca::ReporterManager();
  my $credentials = { cert => "t/certs/client_ca1cert.pem",
                      key => "t/certs/client_ca1keynoenc.pem",
                      passphrase => undef,
                      trusted => "t/certs/trusted" };
  $scheduler->setCredentials( $credentials );
  $scheduler->setCheckPeriod( 1 );
  $scheduelr->setDepots( "incas://localhost:$port" );
  my $rc = new Inca::ReporterManager::ReporterCache( "t" );
  $scheduler->setReporterCache( $rc );
  $scheduler->dispatch( $suite );

;

  }
};
is($@, '', "example from line 12");

    undef $main::_STDOUT_;
    undef $main::_STDERR_;

